(function($) {
  // A reference to the generic hasOwnProperty() function.
  var hasOwnProperty = [].hasOwnProperty;

  // This is an IE fix because these will not be found by a for..in statement.
  var unenumerableKeys = "constructor,hasOwnProperty,isPrototypeOf,prototypeIsEnumerable,toLocaleString,toString,valueOf".split(",");
  for(var key, o = {}, i = 0; key = unenumerableKeys[i]; i++) {
    o[key] = i;
    for(key in o) {
      unenumerableKeys.splice(i--, 1);
      delete o[key];
    }
  }

  // The constructor for the new Hashtable class.
  function Hashtable(obj) {
    obj = arguments.length ? obj : {};

    var hash = {}, key, keys = unenumerableKeys.slice(0), i = 0;
    while(key = keys[i++]) {
      !hasOwnProperty.call(obj, key)
        ? hash[key] = obj[key]
        : keys.splice(i, 1);
    }
    for(key in obj) {
      if(hasOwnProperty.call(obj, key)) {
        keys.push(key);
        hash[key] = obj[key];
      }
    }

    var _ = {o : hash, k : keys};
    this._ = function(k) {
      return k === $ ? _ : undefined;
    };

    return this;
  }

  // Reference for quickly defining prototype functions.
  var p = Hashtable.prototype;

  // Returns the item at the specified `key`.
  p.get = function(key) {
    return this._($).o[key];
  };

  // Returns an array of all of the keys in the hash table.
  p.keys = function() {
    return this._($).k.slice(0);
  };

  // Returns a boolean value indicating whether or not the specified `value` is
  // contained within the hash table.
  p.containsValue = p.contains = function(value) {
    var _ = this._($), keys = _.k, obj = _.o, i = keys.length;
    while(--i >= 0) {
      if(obj[keys[i]] === value) {
        return true;
      }
    }
    return false;
  };

  // Returns a boolean value indicating whether or not the specified `key` is
  // contained within the hash table.
  p.containsKey = function(key) {
    var keys = this._($).k, i = keys.length;
    while(--i >= 0) {
      if(keys[i] == key) {
        return true;
      }
    }
    return false;
  };

  // Clear the hash table and return the object indexed by the keys that
  // contains all of the items.
  p.clear = function() {
    var _ = this._($), ret = _.o;
    _.k = [];
    _.o = {};
    return ret;
  };

  // Get an array of all of the items.
  p.elements = function() {
    var elems = [],
        _ = this._($),
        keys = _.k,
        i = 0,
        len = keys.length,
        obj = _.o;
    while(i < len) {
      elems.push(obj[keys[i++]]);
    }
    return elems;
  };

  // Get the amount of items.
  p.size = function() {
    return this._($).k.length;
  };

  // Removes the item at the specified key and returns the item that is removed.
  p.remove = function(key) {
    var _ = this._($), keys = _.k, obj = _.o, i = keys.length;
    while(--i >= 0) {
      if(keys[i] == key) {
        var ret = obj[keys.splice(i, 1)[0]];
        delete obj[key];
        break;
      }
    }
    return ret;
  };

  // Puts one or more items in this hash table, returning the previous value(s).
  p.put = function(key, value) {
    var _ = this._($), keys = _.k, obj = _.o;
    // If only one value is to be added...
    if(arguments.length - 1) {
      if(hasOwnProperty.call(obj, key)) {
        ret = obj[key];
      }
      else {
        keys.push(key);
      }
      obj[key] = value;
    }
    // If all of the key-value pairs in the specified object are to be added...
    else {
      var ret = {},
          objNew = key,
          newKeys = (new Hashtable(objNew)).keys(),
          i = newKeys.length;
      while(--i >= 0) {
        _ = newKeys[i];
        if(hasOwnProperty.call(obj, _)) {
          ret[_] = obj[_];
        }
        else {
          keys.push(_);
        }
        obj[_] = objNew[_];
      }
    }
    return ret;
  };

  // Makes a shallow copy of the Hashtable.
  p.clone = function() {
    return new Hashtable(this._($).o);
  };

  // Returns true if no items in the hash table, otherwise false is returned.
  p.isEmpty = function() {
    return !this._($).k.length;
  };

  // Returns true if the specified hash table has all of the same key-value
  // pairs, otherwise returns false.
  p.equals = function(hashTable2) {
    var t, i, _ = this._($), o = _.o, k = _.k,
        _2 = hashTable2._($), o2 = _2.o, k2 = _2.k;
    if(k.length != (i = k2.length)) {
      return false;
    }
    for(;--i >= 0 && hasOwnProperty.call(o, t = k2[i]) && o[t] === o2[t];);
    return i < 0;
  };

  // Executes the specified function for each item, sending the specified
  // objThis object (if not specified uses the global object) as the context or
  // `this` object, the item as the first parameter, the key as the second
  // parameter, and this hash table as the final parameter.  Returns a reference
  // to this hash table so that a chained call can occur if desired.
  p.forEach = function(callback, objThis) {
    var ret, _ = this._($), o = _.o, k = _.k;
    for(var i = 0, len = k.length; i < len; i++) {
      callback.call(objThis, o[k[i]], k[i], this);
    }
    return this;
  };

  // If Hashtable is already defined in the global context, save a reference to
  // that definition as the `__OLD__` property of the new Hashtable class that
  // will be defined in the global context.
  if(hasOwnProperty.call(this, "Hashtable")) {
    Hashtable.__OLD__ = this.Hashtable;
  }
  this.Hashtable = Hashtable;
})({});